var GenoType = function()
{
	var gt = {};
	var debug = 0;
	var subtypeAlignedSequences = null;
	var alignedSequences;
	
	// Page initialization
	gt.initPage = function()
	{
		
		// Step 1
		Buttons.addClickHandler('btnSelectAlignedSequences', gt.selectAlignedSequences);
		Events.addEventHandler('alignedSequences', 'base-selected', gt.setAlignedSequenceCallback);
		Events.addEventHandler('step-1', 'wizard-validate', gt.validateStep1);

		// Step 2
		Events.addEventHandler('step-2', 'wizard-initialize', gt.initializeStep2);
		
		// Navigation
		Buttons.addClickHandler('gocancel', Wizard.cancelWizard);
		Buttons.addClickHandler('gorestart', Wizard.restartWizard);
		Buttons.addClickHandler('gonext', Wizard.goNextOnClick);
		Buttons.addClickHandler('goregister', Wizard.goRegister);
			
		// Final registration
		Events.addEventHandler('wizard', 'wizard-submit', gt.submit);

		Wizard.showLoadingAnimation('Loading bioassays...');
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetAlignedSequencesForGenotypeQc';
		Wizard.asyncJsonRequest(url, gt.initializeStep1);
	}
	
	// --- Step 1 -----------------------------------
	gt.initializeStep1 = function(response)
	{
		var alignedSequences = response.alignedSequences;
		var frm = document.forms['reggie'];	

		Doc.element('alreadyChecked').innerHTML = response.existingVcfCount;
		
		if (alignedSequences != null && alignedSequences.length > 0)
		{
			var firstPlate = null;
			for (var asNo=0; asNo < alignedSequences.length; asNo++)
			{
				var seq = alignedSequences[asNo];
				var lib = seq.lib;
				var isYellow = lib.specimen && lib.specimen.YellowLabel != null;
				var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
				if (firstPlate == null && libPlate != null) firstPlate = libPlate.name;
				
				var name = (asNo+1) + ': ';
				if (libPlate) name += libPlate.name + ' - ';
				name += seq.name;
				
				var option = new Option(name, seq.id, false, libPlate != null && libPlate.name==firstPlate);
				if (isYellow) option.className = 'yellow';
				option.alignedSequences = seq;
				frm.alignedSequences.options[frm.alignedSequences.length] = option;
			}
		}
		else
		{
			Wizard.setInputStatus('alignedSequences', 'invalid', 'No new alignments available for processing');
		}
		
		Doc.show('step-1');
		Doc.show('gonext');

		frm.alignedSequences.focus();
	}
	
	gt.validateStep1 = function(event)
	{
		Wizard.setInputStatus('alignedSequences', '', '');
		var frm = document.forms['reggie'];
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) return;
		}
		Wizard.setInputStatus('alignedSequences', 'invalid', 'Select at least one item');
		event.preventDefault();
	}
	
	gt.selectAlignedSequences = function()
	{
		var frm = document.forms['reggie'];
		if (frm.alignedSequences.disabled) return;
		
		if (subtypeAlignedSequences == null)
		{
			subtypeAlignedSequences = Reggie.getSubtypeInfo('ALIGNED_SEQUENCES');
		}
		
		var url = '&resetTemporary=1';
		url += '&tmpfilter:INT:itemSubtype='+subtypeAlignedSequences.id;
		Dialogs.selectItem('DERIVEDBIOASSAY', 'alignedSequences', 1, url);
	}

	gt.setAlignedSequenceCallback = function(event)
	{
		var ms = event.detail;
		
		var frm = document.forms['reggie'];
		var alignedSequences = frm.alignedSequences;
		for (var i = 0; i < alignedSequences.length; i++)
		{
			if (alignedSequences[i].value == ms.id)
			{
				alignedSequences[i].selected = true;
				Wizard.setInputStatus('alignedSequences');
				return;
			}
		}

		var option = new Option(ms.name, ms.id);
		option.alignedSequences = ms;
		option.selected = true;
		alignedSequences[alignedSequences.length] = option;
		Wizard.setInputStatus('alignedSequences');
	}

	gt.initializeStep2 = function()
	{
		var frm = document.forms['reggie'];
		
		var submitInfo = {};
		var alignedSequences = [];
		submitInfo.alignedSequences = alignedSequences;
			
		for (var asNo = 0; asNo < frm.alignedSequences.length; asNo++)
		{
			if (frm.alignedSequences[asNo].selected) 
			{
				var as = {};
				as.id = frm.alignedSequences[asNo].alignedSequences.id;
				alignedSequences[alignedSequences.length] = as;
			}
		}

		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GenotypeQcCheck';
		Wizard.showLoadingAnimation('Comparing genotypes...', 'genotypes-progress');
		Wizard.asyncJsonRequest(url, gt.checkCompleted, 'POST', JSON.stringify(submitInfo));
	}
	
	gt.checkCompleted = function(response)
	{
		alignedSequences = response.alignedSequences;
		
		var html = '<table id="compareTable">';
		html += '<thead class="bg-filled-100">';
		html += '<tr>';
		html += '<th></th>';
		html += '<th class="dottedleft"></th>';
		html += '<th colspan="2">ALIGNED_PAIRS</th>';
		html += '<th  class="dottedleft" colspan="3">Genotypes</th>';
		html += '<th class="dottedleft" colspan="3">Comparisons</th>';
		html += '<th class="dottedleft" colspan="2"></th>';
		html += '<th class="dottedleft" colspan="2">Actions</th>';
		html += '<th class="dottedleft"></th>';
		html += '</tr>';
		html += '<tr>';
		html += '<th>Alignment</th>';
		html += '<th class="dottedleft">Libplate</th>';
		html += '<th>(millions)</th>';
		html += '<th>Dup.</th>';
		html += '<th class="dottedleft">Total</th>';
		html += '<th>HET</th>';
		html += '<th class="icon-col"></th>';
		html += '<th class="dottedleft">Total</th>';
		html += '<th>Same PAT</th>';
		html += '<th>Skipped</th>';
		html += '<th class="dottedleft warning-col"></th>';
		html += '<th>Warnings and messages</th>';
		html += '<th class="dottedleft">Disable</th>';
		html += '<th>Flag</th>';
		html += '<th class="dottedleft">Comment</th>';
		html += '</tr>';
		html += '</thead>';
		html += '<tbody>';
		
		var htmlNoWarn = '';
		var htmlVerified = '';
		
		for (var aNo = 0; aNo < alignedSequences.length; aNo++)
		{
			var aligned = alignedSequences[aNo];
			var lib = aligned.lib;
			var libPlate = lib.bioWell ? lib.bioWell.bioPlate : null;
			var alignedReads = aligned.ALIGNED_PAIRS == null ? '-' : Reggie.formatNumber(aligned.ALIGNED_PAIRS/1000000, null, 2);
			var duplication = aligned.FRACTION_DUPLICATION == null ? '-' : Reggie.formatNumber(100*aligned.FRACTION_DUPLICATION, '%', -1);

			var cmp = aligned.compare;
			var vcf = cmp.vcf;
			var disableChecked = cmp.recommendDisable ? "checked" : "";
			var flagChecked = cmp.recommendFlag ? "checked" : "";
			
			var tmp = '<tr class="highlight">';
			tmp += '<td>'+Strings.encodeTags(aligned.name)+'</td>';
			tmp += '<td class="dottedleft">'+(libPlate ? Strings.encodeTags(libPlate.name) : '-')+'</td>';
			tmp += '<td>'+alignedReads+'</td>';
			tmp += '<td>'+duplication+'</td>';
			tmp += '<td class="dottedleft">'+vcf.gtCount+'</td>';
			tmp += '<td>'+vcf.hetCount+'</td>';
			tmp += '<td class="icon-col">';
			tmp += '<span class="link vcf-link" data-file-id="'+vcf.fileId+'" data-item-id="'+aligned.id+'" title="View genotype statistics"><img src="../images/vcf_file.png"></span>';
			tmp += '</td>';
			tmp += '<td class="dottedleft">'+cmp.total+'</td>';
			tmp += '<td>'+(cmp.samePat || '')+'</td>';
			tmp += '<td>'+(cmp.skipped || '')+'</td>';
			tmp += '<td class="dottedleft icon-col">'
			if (cmp.warnings.length == 0)
			{
				tmp += '<img src="../images/ok.png">';
			}
			else
			{
				tmp += '<img src="../images/warning.png">';
			}
			tmp += '</td>';
			tmp += '<td class="warning-col">';
			for (var wNo = 0; wNo < cmp.warnings.length; wNo++)
			{
				var w = cmp.warnings[wNo];
				tmp += '<div class="warning-msg">'+Strings.encodeTags(w.message);
				if (w.id)
				{
					if (w.flagged) tmp += ' <img src="../images/flag.png" title="This item is already flagged">';
					tmp += ' <span class="link vcf-link" data-file-id="'+vcf.fileId+'" data-item-id="'+aligned.id+'"';
					tmp += ' data-item-id2="'+w.id+'" data-file-id2="'+w.vcf.fileId+'"';
					tmp += ' title="Compare genotypes"><img src="../images/vcf_file.png" title=""></span>';
				}
				tmp += '</div>';
			}
			for (var vNo = 0; vNo < cmp.verified.length; vNo++)
			{
				var v = cmp.verified[vNo];
				tmp += '<div class="warning-msg">'+Strings.encodeTags(v.message);
				if (v.id)
				{
					if (v.flagged) tmp += '<img src="../images/flag.png" title="This item is already flagged">';
					tmp += ' <span class="link vcf-link" data-file-id="'+vcf.fileId+'" data-item-id="'+aligned.id+'"';
					tmp += ' data-item-id2="'+v.id+'" data-file-id2="'+v.vcf.fileId+'"';
					tmp += ' title="Compare genotypes"><img src="../images/vcf_file.png"></span>';
				}
				tmp += '</div>';
			}
			tmp += '</td>';
			tmp += '<td class="dottedleft"><input type="checkbox" name="disable.'+aligned.id+'" '+disableChecked+'></td>';
			tmp += '<td><input type="checkbox" name="flag.'+aligned.id+'" '+flagChecked+'></td>';
			tmp += '<td class="dottedleft comment"><input type="text" name="comment.'+aligned.id+'"></td>';
			tmp += '</tr>';
			
			if (cmp.warnings.length == 0 && cmp.verified.length == 0)
			{
				htmlNoWarn += tmp;
			}
			else if (cmp.warnings.length == 0)
			{
				htmlVerified += tmp;
			}
			else
			{
				html += tmp;
			}
		}
		html += htmlVerified;
		html += htmlNoWarn;
		html += '</tbody>';
		html += '</table>';
		Doc.element('comparisons').innerHTML = html;

		// Add click handler to VCF files
		var clickableItems = document.getElementsByClassName('vcf-link');
		for (var i = 0; i < clickableItems.length; i++)
		{
			Events.addEventHandler(clickableItems[i], 'click', gt.vcfLinkOnClick);
		}
		
		Wizard.setCurrentStep(2);
		Doc.show('step-2');
		Doc.show('goregister');
		Doc.show('gocancel');
	}

	gt.vcfLinkOnClick = function(event)
	{
		var fileId = Data.int(event.currentTarget, 'file-id');
		var itemId = Data.get(event.currentTarget, 'item-id');
		var fileId2 = Data.int(event.currentTarget, 'file-id2');
		var itemId2 = Data.get(event.currentTarget, 'item-id2');

		var url = 'view_genotypes.jsp?ID=' + App.getSessionId();
		url += '&fileId='+fileId;
		url += '&itemId='+itemId;
		url += '&fileId2='+fileId2;
		url += '&itemId2='+itemId2;
		
		Dialogs.openPopup(url, 'ViewVcf', 900, 600);
	}

	gt.submit = function()
	{
		var frm = document.forms['reggie'];
		var submitInfo = {};
		var list = [];
		submitInfo.alignedSequences = list;
		
		for (var aNo = 0; aNo < alignedSequences.length; aNo++)
		{
			var aligned = alignedSequences[aNo];
			var cmp = aligned.compare;
				
			var tmp = {};
			tmp.id = aligned.id;
			tmp.disable = frm['disable.'+aligned.id].checked;
			if (frm['flag.'+aligned.id].checked)
			{
				var flag = [];
				tmp.flag = flag;
				flag[flag.length] = aligned.id;
				for (var wNo = 0; wNo < cmp.warnings.length; wNo++)
				{
					var w = cmp.warnings[wNo];
					if (w.id) flag[flag.length] = w.id;
				}
			}
			tmp.comment = frm['comment.'+aligned.id].value;
			
			list[list.length] = tmp;
		}
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=RegisterQcCheck';
		Wizard.showLoadingAnimation('Registering...');
		Wizard.asyncJsonRequest(url, gt.submissionResults, 'POST', JSON.stringify(submitInfo));
	}
	
	gt.submissionResults = function(response)
	{
		Wizard.showFinalMessage(response.messages);
		Doc.show('gorestart');
	}

	return gt;
}();

Doc.onLoad(GenoType.initPage);

