var ViewGenotypes = function()
{
	var vgt = {};
	var debug = 0;
	
	vgt.initPage = function()
	{
		Buttons.addClickHandler('close', App.closeWindow);
		
		var url = '../Genotype.servlet?ID='+App.getSessionId();
		url += '&cmd=GetVcfStatistics';
		url += '&fileId='+Data.int('page-data', 'file-id');
		url += '&fileId2='+Data.int('page-data', 'file-id2');
		url += '&itemId='+Data.int('page-data', 'item-id');
		url += '&itemId2='+Data.int('page-data', 'item-id2');
		
		Wizard.showLoadingAnimation('Loading VCF file...');
		Wizard.asyncJsonRequest(url, vgt.onFilesLoaded);
	}
	
	vgt.onFilesLoaded = function(response)
	{
		var alignment = response.alignment;
		var alignment2 = response.alignment2;
		var snpData = response.snpData;

		var compareMode = alignment2 != null;

		var vcfData = alignment.vcfData;
		var vcfData2 = compareMode ? alignment2.vcfData : null;
				
		var html = '';
		html += '<tr class="bottomborder topborder">';
		html += '<th>Id</th>';
		html += '<th>Ref</th>';
		html += '<th>Alt</th>';
		html += '<th class="dottedleft"></th>';
		
		var tmp = '<th title="Genotype">GT</th>';
		tmp += '<th title="Genotype Quality">GQ</th>';
//		tmp += '<th title="Approximate read depth (reads with MQ=255 or with bad mates are filtered)">DP</th>';
		tmp += '<th title="Allelic depths for the ref and alt alleles in the order listed">AD</th>';
		
		html += tmp;
		if (compareMode) 
		{
			html += '<th class="dottedleft"><input type="checkbox" id="hideMatches" title="Show only mismatches"></th>';
			html += tmp;
		}
		
		html += '</tr>';
		vcfData.numWithLowGQ = 0;
		vcfData.numWithGQ99 = 0;
		if (compareMode)
		{
			vcfData2.numWithLowGQ = 0;
			vcfData2.numWithGQ99 = 0;
		}
		for (var snpNo = 0; snpNo < snpData.length; snpNo++)
		{
			var e = snpData[snpNo];
			var snp = e.snp;
			var gt = e.gt;
			var gt2 = e.gt2;
			
			var mismatch = compareMode && gt2 && gt && gt.genoType != gt2.genoType;
			
			html += '<tr class="snp highlight '+(mismatch?'mismatch':'match')+'">';
			html += '<td title="'+Strings.encodeTags(snp.chromosome+':'+snp.position)+'">'+Strings.encodeTags(snp.id)+'</td>';
			html += '<td>'+Strings.encodeTags(snp.ref)+'</td>';
			html += '<td>'+Strings.encodeTags(snp.alt)+'</td>';
			html += vgt.gtToHtml(gt, vcfData, 'gt1');
			if (compareMode) html += vgt.gtToHtml(gt2, vcfData2, 'gt2');
			html += '</tr>';
		}
		Doc.element('snp-list').innerHTML = html;

		Doc.element('gtCount').innerHTML = vgt.getGtCount(vcfData);
		Doc.element('avgGQ').innerHTML = vgt.getAvgGQ(vcfData);
		Doc.element('alignedPairs').innerHTML = vgt.getAlignedPairs(alignment);
		if (compareMode)
		{
			Doc.element('gtCount2').innerHTML = vgt.getGtCount(vcfData2);
			Doc.element('avgGQ2').innerHTML = vgt.getAvgGQ(vcfData2);
			Doc.element('alignedPairs2').innerHTML = vgt.getAlignedPairs(alignment2);
			
			Doc.element('libPlate').innerHTML = Strings.encodeTags(vgt.getLibPlate(alignment.lib));
			Doc.element('libPlate2').innerHTML = Strings.encodeTags(vgt.getLibPlate(alignment2.lib));

			Doc.element('qiacube').innerHTML = vgt.getQiacube(alignment);
			Doc.element('qiacube2').innerHTML = vgt.getQiacube(alignment2);

			Doc.element('partitionDate').innerHTML = Reggie.reformatDate(alignment.PARTITION_DATE);
			Doc.element('partitionDate2').innerHTML = Reggie.reformatDate(alignment2.PARTITION_DATE);

			Doc.element('pat').innerHTML = Strings.encodeTags(alignment.patientName);
			Doc.element('pat2').innerHTML = Strings.encodeTags(alignment2.patientName);
			
			Events.addEventHandler('hideMatches', 'change', vgt.showOrHideMatches);
		}
		
		Doc.show('sample-summary', 'table');
	}
	
	
	vgt.gtToHtml = function(gt, vcfData, cls)
	{
		var html = '';
		if (gt)
		{
			var lowGQ = gt.GQ < 50;
			if (lowGQ) vcfData.numWithLowGQ++;
			if (gt.GQ == 99) vcfData.numWithGQ99++;
			if (lowGQ) cls += ' lowgq';
			html += '<td class="dottedleft '+cls+'">'+Strings.encodeTags(gt.genoType)+'</td>';
			html += '<td class="'+cls+'">'+Strings.encodeTags(gt.GT)+'</td>';
			html += '<td class="'+cls+'">'+gt.GQ+'</td>';
//			html += '<td class="'+cls+'">'+gt.DP+'</td>';
			html += '<td class="'+cls+'">'+Strings.encodeTags(gt.AD)+'</td>';
		}
		else
		{
			html += '<td colspan="4" class="dottedleft">-</td>';
		}
		return html;
	}
	
	vgt.getGtCount = function(vcfData)
	{
		var gtCount = '';
		gtCount += vcfData.gtCount;
		gtCount += ' (' + (vcfData.homRefCount+vcfData.homAltCount) + ' HOM';
		gtCount += ', '+vcfData.hetCount + ' HET';
		if (vcfData.badCount) gtCount += ', ' + vcfData.badCount + ' BAD';
		gtCount += ')';
		return gtCount;
	}
	
	vgt.getAvgGQ = function(vcfData)
	{
		var avgGq = (vcfData.sumHomGQ+vcfData.sumHetGQ) / vcfData.gtCount;
		var html = Numbers.formatNumber(avgGq, 0);
		html += ' (' + vcfData.numWithGQ99 + ' with GQ=99, '+vcfData.numWithLowGQ + ' with GQ&lt;50)';
		return html;
	}

	vgt.getLibPlate = function(lib)
	{
		var well = lib.bioWell;
		var libPlate = well ? well.bioPlate : null;
		
		return libPlate ? libPlate.name + ' ' + well.location : '';
	}
	
	vgt.getQiacube = function(alignment)
	{
		var html = Reggie.reformatDate(alignment.QIACUBE_DATE);
		
		html += ' #' + alignment.QIACUBE_RUN_NO;
		html += ' @' + alignment.QIACUBE_POSITION;
		
		return html;
	}
	
	vgt.getAlignedPairs = function(alignment)
	{
		var html = alignment.ALIGNED_PAIRS == null ? '-' : Reggie.formatNumber(alignment.ALIGNED_PAIRS/1000000, 'M', 2);
		if (alignment.FRACTION_DUPLICATION != null)
		{
			html += ' (' + Reggie.formatNumber(100*alignment.FRACTION_DUPLICATION, '%', -1) + ' dup)';
		}
		return html;
	}
	
	vgt.showOrHideMatches = function()
	{
		Doc.addOrRemoveClass('snp-list', 'hidematches');
	}
	
	return vgt;
}();

Doc.onLoad(ViewGenotypes.initPage);
